package org.edg.wl.dgas.dgassim;

import java.util.ArrayList;
import org.edg.wl.dgas.dgassim.fabric.*;

public class Job {

    private int dgJobId;
    private static final int PROTOTYPE = -1;

    private int jobType;

    private int compEnergy;
    private int minCPUPower;

    private int execCEId; // id of executing CE.
    private static final int NO_EXEC_CE = -1;

    private int MIN_JOB_COMP_EN, MAX_JOB_COMP_EN;
    private int MIN_JOB_CPU_REQ, MAX_JOB_CPU_REQ;

    private ArrayList requiredQTypes;

    private int price;

    Job(int type, int minCompEn, int maxCompEn, int minCPUReq, int maxCPUReq,
	ArrayList reqQT) {

	dgJobId = PROTOTYPE;
	jobType = type;
	execCEId = NO_EXEC_CE;

	compEnergy =
	    (int) ((double)(maxCompEn - minCompEn)*(double)Math.random() + 0.5)
	          + minCompEn;

	MIN_JOB_COMP_EN = minCompEn;
	MAX_JOB_COMP_EN = maxCompEn;
	MIN_JOB_CPU_REQ = minCPUReq;
	MAX_JOB_CPU_REQ = maxCPUReq;

	minCPUPower =
	    (int) ((double)(maxCPUReq - minCPUReq)*(double)Math.random() + 0.5)
	          + minCPUReq;

	requiredQTypes = reqQT;
    }

    public int dgJobId() {
	return dgJobId;
    }

    private void setDgJobId(int id) {
	dgJobId = id;
    }

    public int jobType() {
	return jobType;
    }	    

    public void setExecutingCE(int ceID) {
	execCEId = ceID;
    }

    public int executingCE() {
	return execCEId;
    }

    public int compEnergy() {
	return compEnergy;
    }

    public int minCPUPower() {
	return minCPUPower;
    }

    public boolean qTypeAccepted(String qType) {
	if (requiredQTypes == null) // default, accept all
	    return true;

	return requiredQTypes.contains(qType);
    }

    // may be overwritten by derived classes!!!
    public boolean matchCE(ComputingElement ce) {

	if ( ce.getCPUPower() >= minCPUPower() // min. "quality" req.
	     && qTypeAccepted(ce.getQTypeIdString()) // queue type ok
	     && ce.jobMayBeQueued(this) // job still can be queued to CE.
	     ) {

	    return true;
	}

	return false;
    }

    public int getJobPrice() {
	return price;
    }
    public void setJobPrice(int jp) {
	price = jp;
    }

    public Job newInstance(int id) {
	Job newJob = new Job(jobType, MIN_JOB_COMP_EN, MAX_JOB_COMP_EN,
			    MIN_JOB_CPU_REQ, MAX_JOB_CPU_REQ, requiredQTypes);
	newJob.setDgJobId(id);

	return newJob;
    }
}
