package org.edg.wl.dgas.dgassim.monitor;

import java.util.*;
import org.edg.wl.dgas.dgassim.*;
import org.edg.wl.dgas.dgassim.workload.pa.*;
import org.edg.wl.dgas.dgassim.fabric.*;

public class GIS {

    private static int currDgJobId = 0;

    private boolean verbose, debug;

    private ArrayList ceIDs;
    private TreeMap ces; // maps CE IDs to CE references.

    private ArrayList paIDs;
    private TreeMap pas; // maps PA IDs to PA references.
    private TreeMap paStr; // maps PA ID strings to PA references.

    private ArrayList jobTypeIDs;
    private TreeMap jobTypes; // maps job type IDs to job references.

    private TreeMap maxQWTs; // maps queue ID strings max. QWTs.

    public GIS(boolean verbose, boolean debug) {

	ces = new TreeMap();
	ceIDs = new ArrayList();

	pas = new TreeMap();
	paStr = new TreeMap();
	paIDs = new ArrayList();

	jobTypes = new TreeMap();
	jobTypeIDs = new ArrayList();

	maxQWTs = new TreeMap();

	this.verbose = verbose;
	this.debug = debug;

	if (debug)
	    System.out.println("constructor of class GIS");
    }

    public int getCurrDgJobId() {
	return currDgJobId;
    }

    public ArrayList getCEList() {
        ArrayList list = new ArrayList();
	synchronized (this) {
	    for (int i = 0; i < ceIDs.size(); i++) {
		list.add((ComputingElement)ces.get(ceIDs.get(i)));
	    }
	}
        return list;
    }

    public ArrayList getPAList() {
        ArrayList list = new ArrayList();
	synchronized (this) {
	    for (int i = 0; i < paIDs.size(); i++) {
		list.add((PriceAuthority)pas.get(paIDs.get(i)));
	    }
	}
        return list;
    }

    public ArrayList getJobTypeList() {
        ArrayList list = new ArrayList();
	synchronized (this) {
	    for (int i = 0; i < jobTypeIDs.size(); i++) {
		list.add((Job)jobTypes.get(jobTypeIDs.get(i)));
	    }
	}
        return list;
    }

    public ArrayList getCEIDList() {
	return ceIDs;
    }

    public ArrayList getPAIDList() {
	return paIDs;
    }

    public ArrayList getJobTypeIDList() {
	return jobTypeIDs;
    }

    public ComputingElement getCE(int ceID) {
	return (ComputingElement) ces.get(new Integer(ceID));
    }

    public PriceAuthority getPA(int paID) {
	return (PriceAuthority) pas.get(new Integer(paID));
    }
    public PriceAuthority getPA(String paIDStr) {
	return (PriceAuthority) paStr.get(paIDStr);
    }

    public PriceAuthority getCEsPA(int ceId) {
	return ((ComputingElement)ces.get(new Integer(ceId))).getPA();
    }

    public int getPrice(int ceId) {
	return getCEsPA(ceId).getPrice(ceId);
    }

    public Job getJobType(int jobTypeID) {
	return (Job) jobTypes.get(new Integer(jobTypeID));
    }

    public Job createNewJob(int jobTypeID) {
	return getJobType(jobTypeID).newInstance(++currDgJobId);
    }

    public Job createNewJob() {
	// create one at random:
	int jobNum = (int) ( (double)jobTypeIDs.size() *
			                          (double)Math.random() );
	return createNewJob(jobNum);
    }

    public boolean queueIsRegistered(String qTypeStr) {
	return maxQWTs.containsKey(qTypeStr);
    }

    public int getMaxQWT(String qTypeStr) {
	return ((Integer)maxQWTs.get(qTypeStr)).intValue();
    }

    public void registerCE(ComputingElement ce) {
	synchronized (this) {
	    ces.put(new Integer(ce.getId()), ce);
	    ceIDs.add(new Integer(ce.getId()));
	}
    }

    public void registerPA(PriceAuthority pa) {
	synchronized (this) {
	    pas.put(new Integer(pa.getId()), pa);
	    paStr.put(pa.getIdString(), pa);
	    paIDs.add(new Integer(pa.getId()));
	}
    }

    public void registerJobType(Job jType) {
	synchronized (this) {
	    jobTypes.put(new Integer(jType.jobType()), jType);
	    jobTypeIDs.add(new Integer(jType.jobType()));
	}
    }

    public void registerQType(String qTypeStr, int maxQWT) {
	synchronized (this) {
	    maxQWTs.put(new String(qTypeStr), new Integer(maxQWT));
	}
    }

}
